// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>
#include <gsl/gsl_multimin.h>

static int dolightfit;

struct fitvars
{
double ivdark_fitmul;
double ivlight_fitmul;
double chargedark_fitmul;
double chargelight_fitmul;
double rbi_fitmul;
double banderror_fitmul;
double muerror_fitmul;
double simplexmul;
int simplexreset;
int fitvars;
int dofit[20];
};

static struct fitvars fitconfig;

double read_value(char *file,int skip,int line)
{
FILE *in;
char buf[1000];
double value;
in=fopen(file,"r");
int l=0;

do
{
l++;
	fscanf(in,"%s",buf);

	if (l==line)
	{
		sscanf((buf+skip),"%le\n",&value);
		break;
	}


}while (!feof(in));

fclose(in);
return value;
}


void edit_file(char *in_name,char *front,int line,double value)
{
FILE *in;
FILE *out;
char buf[400];
in=fopen(in_name,"r");
out=fopen("temp.tmp","w");
int l=0;

do
{
l++;
	fscanf(in,"%s",buf);
	if ((!feof(in)))
	{
		if (l!=line)
		{
			fprintf(out,"%s\n",buf);
		}else
		{
			fprintf(out,"%s%le\n",front,value);
		}
	}
}while (!feof(in));

fclose(in);
fclose(out);
char command[1000];
sprintf(command,"mv temp.tmp %s",in_name);
system(command);
}

void copy_file(char *out_name, char *in_name,char *front,int line,double value)
{
FILE *in;
FILE *out;
char buf[400];
in=fopen(in_name,"r");
out=fopen(out_name,"w");
int l=0;

do
{
l++;
	fscanf(in,"%s",buf);
	if ((!feof(in)))
	{
		if (l!=line)
		{
			fprintf(out,"%s\n",buf);
		}else
		{
			fprintf(out,"%s%le\n",front,value);
		}
	}
}while (!feof(in));

fclose(in);
fclose(out);
}

double get_charge_error(char *orig_file)
{
printf("get iv error\n");
int i;
double error=0.0;
struct istruct exp;
struct istruct iv;
int fit_len=get_file_len(orig_file)-1;
int ivexternal_len=get_file_len("./charge_dark.dat")-1;
inter_load(&exp,orig_file,fit_len,1,2);
inter_load(&iv,"./charge_dark.dat",ivexternal_len,0,1);
FILE *ferror;
ferror=fopen("./charge_error_dark.dat","w");
//inter_print(&exp);
//getchar();
//printf("len = %d %d\n", ivexternal_len,fit_len);
//getchar();
for (i=0;i<iv.len;i++)
{
if (iv.data[i]<0) iv.data[i]=0.0;
iv.data[i]=sqrt(iv.data[i]*iv.data[i]);

}
double max=0.0;
for (i=0;i<exp.len;i++)
{
//printf("%e\n",exp.data[i]);
exp.data[i]=sqrt(exp.data[i]*exp.data[i]);
if (exp.data[i]>max) max=exp.data[i];
}

for (i=0;i<iv.len;i++)
{
if (iv.data[i]>max) max=iv.data[i];
}

for (i=0;i<iv.len;i++)
{
iv.data[i]/=max;
//printf("iv %e %e\n",iv.x[i],iv.data[i]);
}

for (i=0;i<exp.len;i++)
{
exp.data[i]/=max;
//printf("exp %e %e\n",exp.x[i],exp.data[i]);
}
//getchar();



double start=exp.x[1];
if (iv.x[0]>start) start=iv.x[0];
double stop=0.75;
double points=0.0;
double dx=(stop-start)/1000;
double pos=start;
double etemp=0.0;
do
{
etemp=fitconfig.chargedark_fitmul*fabs(inter_get(&iv,pos)-inter_get(&exp,pos));
fprintf(ferror,"%e %e %e %e\n",pos,etemp,inter_get(&iv,pos),inter_get(&exp,pos));
error+=etemp;
points+=1.0;

pos+=dx;
//printf("%e\n",error);
}while(pos<stop);
fclose(ferror);
/*
double points=0.0;
for (i=0;i<ivexternal_len;i++)
{

//printf("%e %e %e\n",iv.x[i],exp.x[0],exp.x[fit_len-3]);
//getchar();
	if ((iv.x[i]>exp.x[0])&&(iv.x[i]<exp.x[fit_len-3]))
	{
		fprintf(ferror,"%e %e\n",iv.x[i],fabs(iv.data[i]-inter_get(&exp,iv.x[i])));
		error+=fabs(iv.data[i]-inter_get(&exp,iv.x[i]));
		points+=1.0;
		//printf("yes\n");
	}

}
fclose(ferror);
*/
inter_free(&exp);
inter_free(&iv);
//printf("return = %e\n",error);
//getchar();
return error/points;
}



double get_charge_error_light(char *orig_file)
{
printf("get iv error\n");
int i;
double error=0.0;
struct istruct exp;
struct istruct iv;
int fit_len=get_file_len(orig_file)-1;
int ivexternal_len=get_file_len("./charge_light.dat")-1;
inter_load(&exp,orig_file,fit_len,1,2);
inter_load(&iv,"./charge_light.dat",ivexternal_len,0,1);
FILE *ferror;
ferror=fopen("./charge_error_light.dat","w");
//inter_print(&exp);
//getchar();
//printf("len = %d %d\n", ivexternal_len,fit_len);
//getchar();
for (i=0;i<iv.len;i++)
{
if (iv.data[i]<0) iv.data[i]=0.0;
iv.data[i]=sqrt(iv.data[i]*iv.data[i]);

}
double max=0.0;
for (i=0;i<exp.len;i++)
{
//printf("%e\n",exp.data[i]);
exp.data[i]=sqrt(exp.data[i]*exp.data[i]);
if (exp.data[i]>max) max=exp.data[i];
}

for (i=0;i<iv.len;i++)
{
if (iv.data[i]>max) max=iv.data[i];
}

for (i=0;i<iv.len;i++)
{
iv.data[i]/=max;
//printf("iv %e %e\n",iv.x[i],iv.data[i]);
}

for (i=0;i<exp.len;i++)
{
exp.data[i]/=max;
//printf("exp %e %e\n",exp.x[i],exp.data[i]);
}
//getchar();

double start=exp.x[1];
if (iv.x[0]>start) start=iv.x[0];
double stop=0.7;
double points=0.0;
double dx=(stop-start)/1000;
double pos=start;
double etemp=0.0;
do
{
etemp=fitconfig.chargelight_fitmul*fabs(inter_get(&iv,pos)-inter_get(&exp,pos));
fprintf(ferror,"%e %e %e %e\n",pos,etemp,inter_get(&iv,pos),inter_get(&exp,pos));
error+=etemp;
points+=1.0;

pos+=dx;
//printf("%e\n",error);
}while(pos<stop);
fclose(ferror);

inter_free(&exp);
inter_free(&iv);
//printf("return = %e\n",error);
//getchar();
return error/points;
}


double get_rbi_error()
{
char orig_file[200];
strcpy(orig_file,"./chris/k.dat");
printf("get iv error\n");
int i;
double error=0.0;
struct istruct exp;
struct istruct iv;
int fit_len=get_file_len(orig_file)-1;
int ivexternal_len=get_file_len("./rbi.dat")-1;
inter_load(&exp,orig_file,fit_len,0,1);
inter_load(&iv,"./rbi.dat",ivexternal_len,0,1);
FILE *ferror;
ferror=fopen("./rbi_error.dat","w");
//inter_print(&exp);
//getchar();
//printf("len = %d %d\n", ivexternal_len,fit_len);
//getchar();
for (i=0;i<iv.len;i++)
{
if (iv.data[i]<0) iv.data[i]=0.0;
iv.data[i]=sqrt(iv.data[i]*iv.data[i]);

}
double max=0.0;
for (i=0;i<exp.len;i++)
{
//printf("%e\n",exp.data[i]);
exp.data[i]=sqrt(exp.data[i]*exp.data[i]);
if (exp.data[i]>max) max=exp.data[i];
}

for (i=0;i<iv.len;i++)
{
if (iv.data[i]>max) max=iv.data[i];
}

for (i=0;i<iv.len;i++)
{
iv.data[i]/=max;
//printf("%e\n",iv.data[i]);
}

for (i=0;i<exp.len;i++)
{
exp.data[i]/=max;
//printf("%e\n",exp.data[i]);
}
//getchar();
double start=exp.x[0];
if (iv.x[0]>start) start=iv.x[0];
double stop=exp.x[exp.len-1];
double points=0.0;
double dx=(stop-start)/1000;
double pos=start;
double etemp=0.0;
do
{
etemp=fitconfig.rbi_fitmul*fabs(inter_get(&iv,pos)-inter_get(&exp,pos));
fprintf(ferror,"%e %e %e %e\n",pos,etemp,inter_get(&iv,pos),inter_get(&exp,pos));
error+=etemp;
points+=1.0;

pos+=dx;
//printf("%e\n",error);
}while(pos<stop);

fclose(ferror);
inter_free(&exp);
inter_free(&iv);









//printf("return = %e\n",error);
//getchar();
return error/points;
}

int uptodate=FALSE;

double get_iv_error(char *orig_file)
{
int i;
double error=0.0;
struct istruct exp;
struct istruct iv;
int fit_len=get_file_len(orig_file)-1;
int ivexternal_len=get_file_len("./ivexternal_dark.dat")-1;
inter_load(&exp,orig_file,fit_len,0,2);
inter_load(&iv,"./ivexternal_dark.dat",ivexternal_len,0,1);
FILE *ferror;
ferror=fopen("./iv_error_dark.dat","w");
//inter_print(&exp);
//getchar();
//printf("len = %d %d\n", ivexternal_len,fit_len);
//getchar();
double max=0.0;

for (i=0;i<iv.len;i++)
{
iv.data[i]=log10(sqrt(iv.data[i]*iv.data[i]));

}

for (i=0;i<exp.len;i++)
{
exp.data[i]=log10(sqrt(exp.data[i]*exp.data[i]));
}

for (i=0;i<exp.len;i++)
{
if (exp.data[i]>max) max=exp.data[i];
}

for (i=0;i<iv.len;i++)
{
if (iv.data[i]>max) max=iv.data[i];
}

for (i=0;i<iv.len;i++)
{
iv.data[i]/=max;
}

for (i=0;i<exp.len;i++)
{
exp.data[i]/=max;
}

//inter_save(&exp,"myexp.dat");
//inter_save(&iv,"myiv.dat");

double start=exp.x[1];
if (iv.x[0]>start) start=iv.x[0];
double stop=0.6;
double points=0.0;
double dx=(stop-start)/1000;
double pos=start;
double etemp=0.0;
do
{
etemp=fitconfig.ivdark_fitmul*fabs(inter_get(&iv,pos)-inter_get(&exp,pos));
fprintf(ferror,"%e %e %e %e\n",pos,etemp,inter_get(&iv,pos),inter_get(&exp,pos));
error+=etemp;
points+=1.0;

pos+=dx;
//printf("%e\n",error);
}while(pos<stop);
fclose(ferror);
inter_free(&exp);
inter_free(&iv);
return error/points;
}

double get_dos_error()
{
/*double Ne=0.0;
double Nh=0.0;
double ret;
double n;

Ne=fabs(read_value("dos.inp",0,8));
Nh=fabs(read_value("dos.inp",0,10));
n=log10(Ne/Nh);

ret=1-tanh((2-n)*100/(2));
ret=0.0;*/

return 0.0;
}

double get_mualpha_error()
{
int i;
double error=0.0;
struct istruct n;
struct istruct p;
int nlen=get_file_len("./munfile.dat")-1;
int plen=get_file_len("./mupfile.dat")-1;
inter_load(&n,"./munfile.dat",nlen,0,1);
inter_load(&p,"./mupfile.dat",plen,0,1);
FILE *ferror;
ferror=fopen("./alpha_error.dat","w");
//inter_print(&exp);
//getchar();
//printf("len = %d %d\n", ivexternal_len,fit_len);
//getchar();

double error_tot=0.0;
double points=0.0;
double alpha=0.0;
double beta=0.0;
double mue;
double muh;
alpha=read_value("blom_device.inp",0,121);
beta=read_value("blom_device.inp",0,123);

for (i=0;i<p.len;i++)
{

	if ((p.x[i]>5e21)&&(p.x[i]<8e22))
	{
	mue=alpha*inter_get(&n,p.x[i]);
	muh=beta*p.data[i];
	error=1+tanh((mue-muh)/mue);
	error_tot+=error;
	fprintf(ferror,"%e %e %e %e %e %e\n",p.x[i],error,mue,muh,inter_get(&n,p.x[i]),p.data[i]);
	points+=1.0;
	}

}


fclose(ferror);


inter_free(&n);
inter_free(&p);
//printf("I will return %e %e %e\n",error_tot,points,band_error);
//getchar();
return 0.0;//error_tot/points;
}



double get_band_error()
{
int dostype=read_value("dos.inp",0,2);
double Ee=0.0;
double Eh=0.0;
double ret=0.0;
if (dostype==0)
{
	Ee=read_value("dos.inp",0,38);
	Eh=read_value("dos.inp",0,40);
	ret=1+tanh( (Ee-Eh)*20.0/Ee);
	ret*=fitconfig.banderror_fitmul;
}else
{

	Ee=read_value("dos.inp",0,12);
	Eh=read_value("dos.inp",0,14);
	ret=1+tanh( (Ee-Eh)*5.0/Ee);
	ret*=fitconfig.banderror_fitmul;
}

return ret;
}

double get_mu_error()
{
int i;
double error=0.0;
struct istruct n;
struct istruct p;
int nlen=get_file_len("./munfile.dat")-1;
int plen=get_file_len("./mupfile.dat")-1;
inter_load(&n,"./munfile.dat",nlen,0,1);
inter_load(&p,"./mupfile.dat",plen,0,1);
FILE *ferror;
ferror=fopen("./mu_error.dat","w");
//inter_print(&exp);
//getchar();
//printf("len = %d %d\n", ivexternal_len,fit_len);
//getchar();

double band_error=get_band_error();
double dos_error=get_dos_error();
double error_tot=0.0;
double points=0.0;
for (i=0;i<p.len;i++)
{

	if ((p.x[i]>5e21)&&(p.x[i]<8e22))
	{
	error=fitconfig.muerror_fitmul*(1+tanh((p.data[i]-inter_get(&n,p.x[i]))/p.data[i]));
	error_tot+=error;
	fprintf(ferror,"%e %e %e %e %e %e\n",p.x[i],error,band_error,dos_error,inter_get(&n,p.x[i]),p.data[i]);
	points+=1.0;
	}

}


fclose(ferror);


inter_free(&n);
inter_free(&p);
//printf("I will return %e %e %e\n",error_tot,points,band_error);
//getchar();
return error_tot/points+band_error+dos_error;
}



double get_iv_error_light(char *orig_file)
{
int i;
double error=0.0;
struct istruct exp;
struct istruct iv;
int fit_len=get_file_len(orig_file)-1;
int ivexternal_len=get_file_len("./ivexternal_light.dat")-1;
inter_load(&exp,orig_file,fit_len,0,2);
inter_load(&iv,"./ivexternal_light.dat",ivexternal_len,0,1);
FILE *ferror;
ferror=fopen("./iv_error_light.dat","w");
//inter_print(&exp);
//getchar();
//printf("len = %d %d\n", ivexternal_len,fit_len);
//getchar();

double max=0.0;


for (i=0;i<exp.len;i++)
{
if (exp.data[i]>max) max=exp.data[i];
}

for (i=0;i<iv.len;i++)
{
if (iv.data[i]>max) max=iv.data[i];
}

for (i=0;i<iv.len;i++)
{
iv.data[i]/=max;
}

for (i=0;i<exp.len;i++)
{
exp.data[i]/=max;
}




double start=0.3;
if (exp.x[1]>start) start=exp.x[1];
if (iv.x[0]>start) start=iv.x[0];

double stop=0.7;
double points=0.0;
double dx=(stop-start)/1000;
double pos=start;
double etemp=0.0;
do
{
etemp=fitconfig.ivlight_fitmul*fabs(inter_get(&iv,pos)-inter_get(&exp,pos));
fprintf(ferror,"%e %e %e %e\n",pos,etemp,inter_get(&iv,pos),inter_get(&exp,pos));
error+=etemp;
points+=1.0;

pos+=dx;
//printf("%e\n",error);
}while(pos<stop);
fclose(ferror);


inter_free(&exp);
inter_free(&iv);

return error/points;
}



double last_iv_error;
double last_charge_error;
double last_iv_error_light;
double last_charge_error_light;
double last_rbi_error;
double last_mu_error;
double last_mualpha_error;
double get_all_error()
{
last_iv_error=get_iv_error("./chris/dark.dat");
last_charge_error=get_charge_error("./chris/dark.dat");
if (dolightfit==TRUE)
{
last_iv_error_light=get_iv_error_light("./chris/1.0.dat");
last_charge_error_light=get_charge_error_light("./chris/1.0.dat");
}else
{
last_iv_error_light=0.0;
last_charge_error_light=0.0;
}
last_rbi_error=get_rbi_error();
last_mu_error=get_mu_error();
last_mualpha_error=get_mualpha_error();

printf("%e %e %e %e %e\n",last_iv_error,last_charge_error,last_iv_error_light,last_charge_error_light,last_rbi_error);
return last_iv_error+last_charge_error+last_rbi_error+last_iv_error_light+last_charge_error_light+last_mu_error+last_mualpha_error;

}

double my_f (const gsl_vector *v, void *params)
{
double val;
double e0;
int item=0;
int fitcount=0;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("blom_device.inp","",25,val);		//pcontact
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("blom_device.inp","",29,val);		//ncontact
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("blom_device.inp","",113,val);	//Dphoton
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("blom_device.inp","",121,val);	//alpha
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("blom_device.inp","",123,val);	//beta
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",8,val);			//Ntrape
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",10,val);			//Ntraph
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",12,val);			//Etrape
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",14,val);			//Etraph
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",26,val);			//bandstarte
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",28,val);			//bandstarth
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",34,val);			//mue
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",36,val);			//muh
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",38,val);			//sigmae
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",40,val);			//sigmah
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",42,val);			//Ne
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val = gsl_vector_get(v, item);
edit_file("dos.inp","",44,val);			//Nh
item++;
}
fitcount++;


if (dolightfit==TRUE)
{
edit_file("blom_device.inp","",53,1000.0);
edit_file("blom_device.inp","",57,0.3);
run_simulation("./blom_device.inp");
system("cp ./ivexternal_light.dat ./ivexternal_light.old");
system("cp ./ivexternal.dat ./ivexternal_light.dat");
system("cp ./charge_light.dat ./charge_light.old");
system("cp ./charge.dat ./charge_light.dat");
}

edit_file("blom_device.inp","",53,0.0);
edit_file("blom_device.inp","",57,0.2);
run_simulation("./blom_device.inp");
system("cp ./ivexternal_dark.dat ./ivexternal_dark.old");
system("cp ./ivexternal.dat ./ivexternal_dark.dat");
system("cp ./charge_dark.dat ./charge_dark.old");
system("cp ./charge.dat ./charge_dark.dat");

e0=get_all_error();

return e0;
}


void fit_now()
{

FILE *out;
//out = fopen("./fitlog.dat","w");
//fclose(out);

char name[1000];
FILE *config;
config = fopen("./fit.inp","r");
if (config == NULL)
{
	printf("no file %s\n",name);
	exit(0);
}

fscanf(config,"%s",name);
fscanf(config,"%d",&(dolightfit));
fscanf(config,"%s",name);
fscanf(config,"%lf",&(fitconfig.ivdark_fitmul));
fscanf(config,"%s",name);
fscanf(config,"%lf",&(fitconfig.ivlight_fitmul));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(fitconfig.chargedark_fitmul));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(fitconfig.chargelight_fitmul));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(fitconfig.rbi_fitmul));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(fitconfig.banderror_fitmul));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(fitconfig.muerror_fitmul));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(fitconfig.simplexmul));
fscanf(config,"%s",name);
fscanf(config,"%d",&(fitconfig.simplexreset));

fscanf(config,"%s",name);
int max=17;
int i;
fitconfig.fitvars=0;
for (i=0;i<max;i++)
{
fscanf(config,"%s",name);
fscanf(config,"%d",&(fitconfig.dofit[i]));
if (fitconfig.dofit[i]==TRUE) fitconfig.fitvars+=1;
printf("%d %d\n",i,fitconfig.fitvars);
}
fclose(config);
//getchar();
double val=0.0;
double par[5] = {1.0, 2.0, 10.0, 20.0, 30.0};

size_t iter = 0;
int status;
double size;

const gsl_multimin_fminimizer_type *T =
 gsl_multimin_fminimizer_nmsimplex;//gsl_multimin_fminimizer_nmsimplex;
gsl_multimin_fminimizer *s = NULL;
gsl_vector *ss, *x,*old_vector, *old_shift;
gsl_multimin_function minex_func;



/* Starting point */
x = gsl_vector_alloc (fitconfig.fitvars);
ss = gsl_vector_alloc (fitconfig.fitvars);
old_vector = gsl_vector_alloc (fitconfig.fitvars);
old_shift = gsl_vector_alloc (fitconfig.fitvars);
int item=0;
double shift=fitconfig.simplexmul;
int fitcount=0;
if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("blom_device.inp",0,25);		//pcontact
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;

if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("blom_device.inp",0,29);		//pcontact
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("blom_device.inp",0,113);	//alpha
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("blom_device.inp",0,121);	//alpha
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("blom_device.inp",0,123);	//alpha
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,8);			//Ntrape
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,10);			//Ntraph
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,12);			//Etrape
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,14);			//Etraph
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,26);			//bandstarte
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,28);			//bandstarth
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,34);			//mue
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,36);			//muh
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,38);			//sigmae
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,40);			//sigmah
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,42);			//Ne
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;


if (fitconfig.dofit[fitcount]==TRUE)
{
val=read_value("dos.inp",0,44);			//Nh
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
}
fitcount++;

       /* Initialize method and iterate */
       minex_func.n = item;
       minex_func.f = my_f;
       minex_func.params = par;

       s = gsl_multimin_fminimizer_alloc (T, item);
       gsl_multimin_fminimizer_set (s, &minex_func, x, ss);
     int loop=0;
       do
         {
           iter++;
           status = gsl_multimin_fminimizer_iterate(s);

           if (status)
             break;

           size = gsl_multimin_fminimizer_size (s);
           status = gsl_multimin_test_size (size, 1e-8);

           if (status == GSL_SUCCESS)
             {
               printf ("converged to minimum at\n");
             }
printf("---------------------------------------------------------------------\n");
out = fopen("./fitlog.dat","a");
gsl_vector_sub (old_vector, s->x);
gsl_vector_sub (old_shift, ss);
           printf ("%d %le %le\n", (int)iter, s->fval, size);
 		fprintf (out,"%d %le  %le ", (int)iter, s->fval, size);
		fprintf(out,"%e %e %e %e %e %e %e ",last_iv_error,last_charge_error,last_iv_error_light,last_charge_error_light,last_rbi_error,last_mu_error,last_mualpha_error);
		for (i=0;i<item;i++)
		{
			printf("%e %e %e %e\n",gsl_vector_get (s->x, i),gsl_vector_get(ss, i),gsl_vector_get (old_vector, i),gsl_vector_get (old_shift, i));
			fprintf(out,"%e ",gsl_vector_get (s->x, i));
		}
			printf("\n");
			fprintf(out,"\n");

fclose(out);
printf("---------------------------------------------------------------------\n");
gsl_vector_memcpy (old_vector, s->x);
gsl_vector_memcpy (old_shift, ss);
	loop++;
	if (loop>fitconfig.simplexreset)
	{
	break;
	}
         }
       while (status == GSL_CONTINUE);

       gsl_vector_free(x);
       gsl_vector_free(ss);
       gsl_multimin_fminimizer_free (s);
       gsl_vector_free (old_vector);
       gsl_vector_free (old_shift);


}

