// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include "hist.h"
#include <math.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/wait.h>

//#define test_dist_n
//#define test_dist_p

double pick[1000];
double pick_x[1000];
double pick_start=-7;
double pick_stop=0;
double pick_dx;
double draw_Ec=-3.8;
double draw_Ev=-1.0400-3.8;
void pick_init()
{
int i;
pick_dx=(pick_stop-pick_start)/1000.0;
double pos=pick_start;
for (i=0;i<1000;i++)
{
pick[i]=0.0;
pick_x[i]=pos;
pos+=pick_dx;
}

}

void pick_add(double x,double value)
{
int pos;
for (pos=0;pos<1000;pos++)
{
if (pick_x[pos]>x) break;
}
if (pos<0.0) return;
if (pos>999) return;
if (value>pick[pos]) pick[pos]=value;
}

void pick_dump()
{
int i;
FILE *out;
out=fopen("./gaus.dat","w");
double pos=pick_start;
for (i=0;i<1000;i++)
{
fprintf(out,"%e %e\n",pick_x[i],pick[i]);
pos+=pick_dx;
}
fclose(out);
}

void gen_dos_fd_gaus_n(double sigmain,double Nc,double Tstart,double Tstop,int Tsteps,int bands,double dband,int traps,double munrgstop,double mue,double alpha,double xstart,double xsteps,double dx,struct dosconfig *in)
{
printf("Electrons....\n");
FILE *out;
//double xstart=0.0;
//double xsteps=0.0;
//double dx=0.0;
double tstart=0.0;
double tstop=0.0;
double tsteps=0.0;
double dt=0.0;
double xpos=0.0;
double tpos=0.0;
double n;
double *xband;
xband=(double *)malloc(sizeof(double)*(bands+traps));
munrgstop*=q;
int t=0;
int x=0;

double mu=0.0;
double mu_tot=0.0;
out=fopen("./dosn_gau.dat","w");
tstart=Tstart;
tstop=Tstop;
tsteps=Tsteps;
dt=(tstop-tstart)/tsteps;
xpos=xstart;
tpos=tstart;
t=0;
x=0;
double 	dx0=3.0e-3;
fprintf(out,"%d %d %d %d %lf %lf\n",(int)xsteps,(int)tsteps,bands,traps,sigmain,in->edge);

xpos=xstart;
double *xmesh=NULL;
xmesh=(double *)malloc(sizeof(double)*xsteps);
for (x=0;x<xsteps;x++)
{
	dx=dx0;
	if (xpos>0) dx=dx0*10.0;
	fprintf(out,"%e\n",xpos);
	xmesh[x]=xpos;
	xpos+=dx;
}


for (t=0;t<tsteps;t++)
{
	fprintf(out,"%e\n",tpos);
	tpos+=dt;
}

xpos=q*in->edge;

for (x=0;x<(bands+traps);x++)
{
fprintf(out,"%e\n",xpos);
xband[x]=xpos;
xpos+=q*dband;
}


int e=0;
double Esteps=4000;
double *nband=(double *)malloc(sizeof(double)*(bands+traps));
double Estart=-2.0*q;
int band=0;
double Estop=2.0*q;
double dE=(Estop-Estart)/Esteps;
double E=Estart;
double rho;
double sum=0.0;
double sum_w=0.0;
double f=0.0;
tpos=tstart;

FILE *munfile;
munfile=fopen("./munfile.dat","w");
  if (munfile==NULL)
  {
printf("problem\n");
exit(0);
  }

#ifdef test_dist_n
tpos=300;



FILE *plotbands;
plotbands=fopen("./plotbandsn.dat","w");

FILE *plotbandsfree;
plotbandsfree=fopen("./plotbandsfreen.dat","w");




FILE *rod=fopen("./gau_test_n.dat","w");

#endif


double n_tot=0.0;
double sigma=sigmain*q;

#ifdef test_dist_n
int first=FALSE;
#endif

for (t=0;t<tsteps;t++)
{
printf("Electron %d/%d\n",t,(int)tsteps);

	for (x=0;x<xsteps;x++)
	{
		xpos=xmesh[x];
		E=Estart;
		sum=0.0;
		sum_w=0.0;

		for (band=0;band<(bands+traps);band++)
		{
			nband[band]=0.0;
		}

		band=0;
		mu_tot=0.0;
		n_tot=0.0;
		for (e=0;e<(Esteps);e++)
		{
			if (E>xband[band])
			{
				if (band<((bands+traps)-1)) band++;
			}

			//printf("%d %d\n",bands+traps,band);


			f=1.0/(1.0+exp((-xpos+E/q)*q/(kb*tpos)));
			if (in->dostype==dos_gau)
			{
				rho=Nc*(1.0/(sqrt(2.0*pi)*sigma))*exp(-pow(((E)/(sqrt(2.0)*sigma)),2.0));
				//if (E<-0.1*q) gau=0.0;
				n=rho*f*dE;
			}else
			if (in->dostype==dos_fd)
			{
				if (E>0)
				{
					rho=(sqrt(E)/(4.0*pi*pi))*pow((2.0*in->m*m0)/(hbar*hbar),3.0/2.0);
				}else
				{
					rho=0.0;
				}

				n=2.0*rho*f*dE;
			}else
			if (in->dostype==dos_ub)
			{
				if (E>=0)
				{
					rho=Nc*(1.0/(sqrt(2.0*pi)*sigma))*exp(-pow(((E)/(sqrt(2.0)*sigma)),2.0));
				}else
				if (E<0)
				{
					if (E>-1.0*q)
					{
						rho=in->Nt*exp((E+in->Eshift*q)/(in->Et*q));
						//rho+=in->Nt2*exp((E)/(in->Et2*q));
						//printf("%e %e %e\n",in->Nt,in->Et,E);
						//getchar();
					}else
					{
						rho=0.0;
					}
				}

				//if (E<-1.0*q) rho=0.0;

				n=rho*f*dE;
			}else
			if (in->dostype==dos_ub_and_gaus)
			{

				rho=Nc*(1.0/(sqrt(2.0*pi)*sigma))*exp(-pow(((E)/(sqrt(2.0)*sigma)),2.0));

				if (E<0)
				{

						rho+=in->Nt*exp((E+in->Eshift*q)/(in->Et*q));
						//rho+=in->Nt2*exp((E)/(in->Et2*q));
						//printf("%e %e %e\n",in->Nt,in->Et,E);
						//getchar();
				}

				if (E<-1.0*q) rho=0.0;

				n=rho*f*dE;
			}


			if (x==0) pick_add(E/q-3.8,rho);

			sum+=n;
			nband[band]+=n;
			sum_w+=0.0;
			if (E>=xband[traps-1])
			{
				mu=mue;//*exp(alpha*(E-xband[traps-1])/q);
				mu_tot+=mu*n;
				//printf("%e\n",mu);
			//getchar();
				#ifdef test_dist_n
				if (t==0)
				if ((x%10)==0)
				if ((e%2)==0)
				{
					fprintf(plotbandsfree,"%.20le %.20le\n",E,n);
					if (first==FALSE) fprintf(plotbands,"%.20le %.20le\n",E,n);
					first=TRUE;
				}
				#endif

			}else
			{
				#ifdef test_dist_n
				if (t==0)
				if ((x%10)==0)
				if ((e%10)==0)
				{
					fprintf(plotbands,"%.20le %.20le\n",E,n);
				}
				#endif
			}

			n_tot+=n;

			//printf("%d %e %e %e %e %e\n",e,E/q,rho,f,gau,sum);

			E+=dE;
		}

		#ifdef test_dist_n
		fprintf(plotbands,"\n");
		fprintf(plotbandsfree,"\n");
		#endif

//getchar();
		//N=
		//p=N*exp((xpos*q)/(kb*tpos));

		//double N=2.0*pow(((2.0*pi*kb*tpos*m*m0)/(hp*hp)),1.5);
		//printf("N= %e\n",N);
		//getchar();
		//n_tot=N*exp((xpos*q)/(kb*tpos));
		//if (xpos==-1e-3)
		//{
		//	printf("I should be %e\n",n_tot);
		//	getchar();
		//}
		//sum=n_tot;
		//mu_tot=sum*1e-7;
		//nband[0]=0.0;
		//nband[1]=sum;

		//printf("%e %e\n",sum,n_tot);
		//getchar();
		fprintf(out,"%.20le %.20le %.20le ",sum,(3.0/2.0)*kb*tpos,mu_tot/n_tot); //sum_w/sum

		//printf("I want to print %d bands\n",bands+traps);
		for (band=0;band<(bands+traps);band++)
		{
			fprintf(out,"%.20le ",nband[band]);
			//printf("%e ",nband[band]);
			//getchar();
			#ifdef test_dist_n
			fprintf(rod,"%.20le %.20le\n",xband[band]/q,nband[band]);
			#endif
		}
		fprintf(out,"\n");


		#ifdef test_dist_n
		fprintf(rod,"\n");


		#endif
		if (t==0) fprintf(munfile,"%le %le\n",n_tot,mu_tot/n_tot);
		//printf("%.20le\n",sum);
		//getchar();
	}


tpos+=dt;
}
#ifdef test_dist_n
	fclose(rod);
	fclose(plotbands);
	fclose(plotbandsfree);

#endif

fclose(munfile);
fclose(out);
free(xmesh);
free(xband);
return;
}


void gen_dos_fd_gaus_p(double sigmaih, double Nv,double Tstart,double Tstop,int Tsteps,int bands,double dband,int traps,double munrgstop,double muh,double alpha,double xstart,double xsteps,double dx0,struct dosconfig *in)
{
printf("Holes....\n");
FILE *out;
int band;
double tstart=0.0;
double tstop=0.0;
double tsteps=0.0;
double dt=0.0;
double xpos=0.0;
double tpos=0.0;
int t=0;
int x=0;
double N=0.0;
double p=0.0;
double dx;
int e=0;
double Esteps=4000;
double *nband=(double *)malloc(sizeof(double)*(bands+traps));
double Estart=-2.0*q;
double Estop=2.0*q;
double dE=(Estop-Estart)/Esteps;
double E=Estart;
double rho;
double sum=0.0;
double sum_w=0.0;
double f=0.0;
double *xmesh;

double *xband;
xband=(double *)malloc(sizeof(double)*(bands+traps));
munrgstop*=q;



out=fopen("./dosp_gau.dat","w");
FILE *mupfile;
mupfile=fopen("./mupfile.dat","w");

#ifdef test_dist_p

FILE *rod=fopen("./gau_test_p.dat","w");



FILE *plotbands;
plotbands=fopen("./plotbandsp.dat","w");

FILE *plotbandsfree;
plotbandsfree=fopen("./plotbandsfreep.dat","w");
#endif

tstart=Tstart;
tstop=Tstop;
tsteps=Tsteps;
dt=(tstop-tstart)/tsteps;
xpos=xstart;
tpos=tstart;
t=0;
x=0;
//double Efirst=get_first_state(0.1);
double sigma=sigmaih*q;
double n;
double mu_tot;
double p_tot;
double mu;

int first=FALSE;

fprintf(out,"%d %d %d %d %lf %lf\n",(int)xsteps,(int)tsteps, (int)bands,traps,sigmaih,in->edge);

xmesh=(double *)malloc(sizeof(double)*xsteps);
for (x=0;x<xsteps;x++)
{
	dx=dx0;
	//if (xpos<-0.4) dx=dx0*3.0;
	if (xpos>0.0) dx=dx0*10.0;
	fprintf(out,"%e\n",xpos);
	xmesh[x]=xpos;
	xpos+=dx;
}

for (t=0;t<tsteps;t++)
{
	fprintf(out,"%e\n",tpos);
	tpos+=dt;
}

tpos=tstart;

xpos=q*in->edge;

for (x=0;x<(bands+traps);x++)
{
fprintf(out,"%e\n",xpos);
xband[x]=xpos;
xpos+=q*dband;
}


#ifdef test_dist_p
tpos=300;
#endif

for (t=0;t<tsteps;t++)
{
printf("Hole %d/%d\n",t,(int)tsteps);
first=FALSE;
	for (x=0;x<xsteps;x++)
	{
		xpos=xmesh[x];
		E=Estart;
		sum=0.0;
		sum_w=0.0;

		for (band=0;band<(bands+traps);band++)
		{
			nband[band]=0.0;
		}

		band=0;

		mu_tot=0.0;
		p_tot=0.0;

		for (e=0;e<(Esteps);e++)
		{
			if (E>xband[band])
			{
				if (band<((bands+traps)-1)) band++;
			}



			f=1.0/(1.0+exp((-xpos+E/q)*q/(kb*tpos)));
			if (in->dostype==dos_gau)
			{
				rho=Nv*(1.0/(sqrt(2.0*pi)*sigma))*exp(-pow(((E)/(sqrt(2.0)*sigma)),2.0));
				//if (E<-0.1*q) gau=0.0;

				n=rho*f*dE;
			}else
			if (in->dostype==dos_fd)
			{
				if (E>0)
				{
					rho=(sqrt(E)/(4.0*pi*pi))*pow((2.0*in->m*m0)/(hbar*hbar),3.0/2.0);
				}else
				{
					rho=0.0;
				}

				n=2.0*rho*f*dE;
			}else
			if (in->dostype==dos_ub)
			{
				if (E>=0)
				{
					rho=Nv*(1.0/(sqrt(2.0*pi)*sigma))*exp(-pow(((E)/(sqrt(2.0)*sigma)),2.0));
				}else
				if (E<0.0)
				{
					if (E>-1.0*q)
					{
						rho=in->Nt*exp((E+in->Eshift*q)/(in->Et*q));
						//rho+=in->Nt2*exp((E)/(in->Et2*q));
					}else
					{
						rho=0.0;
					}
				}
				//if (E<-1.0*q) rho=0.0;



				n=rho*f*dE;
			}else
			if (in->dostype==dos_ub_and_gaus)
			{
				//if (E>=0)
				//{
					rho=Nv*(1.0/(sqrt(2.0*pi)*sigma))*exp(-pow(((E)/(sqrt(2.0)*sigma)),2.0));
				//}else
				if (E<0.0)
				{
						rho+=in->Nt*exp((E+in->Eshift*q)/(in->Et*q));
						//rho+=in->Nt2*exp((E)/(in->Et2*q));
				}

				if (E<-1.0*q) rho=0.0;



				n=rho*f*dE;
			}


			if (x==0) pick_add(-E/q-4.8,rho);


			//printf("%e %e %e\n",xpos+E/q,dE/q,E/q);
			//getchar();
			//printf("%le %le\n",E/q,n);

			sum+=n;
			nband[band]+=n;

			//check+=dE*gau;
			sum_w+=0.0;//(E-Efirst)*gau*rho*f*dE;
			//printf("%e %e %e %e %e\n",E/q,Efirst/q,(E-Efirst)/q,sum_w/q,sum);
			//getchar();

			if (E>=xband[traps-1])
			{
				//mu=mustart+(alpha-mustart)*(E-xband[traps-1])/(munrgstop-xband[traps-1]);
				mu=muh;//*exp(alpha*(E-xband[traps-1])/q);
				mu_tot+=mu*n;
				//printf("%e %e %e\n",E/q,mu,mu_tot/p_tot);
				//getchar();

				#ifdef test_dist_p
				if (t==0)
				if ((x%10)==0)
				if ((e%10)==0)
				{
					fprintf(plotbandsfree,"%.20le %.20le\n",E,n);
					if (first==FALSE)
					{
						fprintf(plotbands,"%.20le %.20le\n",E,n);
						//printf("---------------------%e\n",E/q);
						//getchar();
					}
					first=TRUE;
				}
				#endif

				//printf("free %e\n",E/q);
				//getchar();


			}else
			{
				#ifdef test_dist_p

				if (t==0)
				if ((x%10)==0)
				if ((e%2)==0)
				{
					fprintf(plotbands,"%.20le %.20le\n",E,n);
				}
				//printf("trap %e\n",E/q);
				//getchar();
				#endif
			}

			p_tot+=n;

			E+=dE;
		}

		//getchar();
		#ifdef test_dist_p
		fprintf(plotbandsfree,"\n");
		fprintf(plotbands,"\n");
		#endif

		//printf("I am = 1 %e\n",check);
		//getchar();
		//N=2.0*pow(((2.0*pi*kb*tpos*m*m0)/(hp*hp)),1.5);
		//p=N*exp((xpos*q)/(kb*tpos));

		N=2.0*pow(((2.0*pi*kb*tpos*in->m*m0)/(hp*hp)),1.5);
		p=N*exp((xpos*q)/(kb*tpos));

		fprintf(out,"%.20le %.20le %.20le ",sum,(3.0/2.0)*kb*tpos,mu_tot/p_tot); //sum_w/sum


		for (band=0;band<(bands+traps);band++)
		{
			fprintf(out,"%.20le ",nband[band]);
			#ifdef test_dist_p
			fprintf(rod,"%.20le %.20le\n",xband[band]/q,nband[band]);
			#endif
		}
		fprintf(out,"\n");
		#ifdef test_dist_p
		fprintf(rod,"\n");
		#endif
//		getchar();
		//printf("%le %le %le %le\n",xpos,sum_w/sum/q,(3.0/2.0)*kb*tpos/q,sum);
		//getchar();

		//printf("%le %le\n",p,sum);
		//getchar();
		//xpos+=dx;

		if (t==0) fprintf(mupfile,"%le %le\n",p_tot,mu_tot/p_tot);

	}

	tpos+=dt;
}

#ifdef test_dist_p
fclose(plotbands);
fclose(plotbandsfree);
fclose(rod);

#endif
fclose(mupfile);
free(xmesh);
free(xband);
fclose(out);
}

void gen_dos_fd_gaus_fd()
{
struct dosconfig confige;
struct dosconfig configh;
pick_init();
FILE *in;
double sigmae;
double sigmah;
double Nc;
double Nv;
double Tstart;
double Tstop;
int Tsteps;
int bands;
double dband;
int traps;

double munrgstop;
double mue;
double muh;
int gendos=TRUE;
double alpha=0.0;

double nxstart;
double nxsteps;
double ndx;

double pxstart;
double pxsteps;
double pdx;

char name[100];
in=fopen ("./dos.inp","r");
if (in==NULL)
{
printf("Density of states config not found\n");
exit(0);
}
fscanf(in,"%s",name);
fscanf(in,"%d",&(confige.dostype));
configh.dostype=confige.dostype;
fscanf(in,"%s",name);
fscanf(in,"%lf",&(confige.m));
fscanf(in,"%s",name);
fscanf(in,"%lf",&(configh.m));


fscanf(in,"%s",name);
fscanf(in,"%lf",&(confige.Nt));
fscanf(in,"%s",name);
fscanf(in,"%lf",&(configh.Nt));
confige.Nt=fabs(confige.Nt);
configh.Nt=fabs(configh.Nt);
fscanf(in,"%s",name);
fscanf(in,"%lf",&(confige.Et));
fscanf(in,"%s",name);
fscanf(in,"%lf",&(configh.Et));
confige.Et=fabs(confige.Et);
configh.Et=fabs(configh.Et);

fscanf(in,"%s",name);
fscanf(in,"%lf",&(confige.Eshift));
fscanf(in,"%s",name);
fscanf(in,"%lf",&(configh.Eshift));




fscanf(in,"%s",name);
fscanf(in,"%d",&gendos);
fscanf(in,"%s",name);
fscanf(in,"%d",&bands);
fscanf(in,"%s",name);
fscanf(in,"%le",&dband);
fscanf(in,"%s",name);
fscanf(in,"%le",&(confige.edge));
fscanf(in,"%s",name);
fscanf(in,"%le",&(configh.edge));
fscanf(in,"%s",name);
fscanf(in,"%d",&traps);
fscanf(in,"%s",name);
fscanf(in,"%le",&munrgstop);

fscanf(in,"%s",name);
fscanf(in,"%le",&mue);
fscanf(in,"%s",name);
fscanf(in,"%le",&muh);
mue=fabs(mue);
muh=fabs(muh);
fscanf(in,"%s",name);
fscanf(in,"%le",&sigmae);
sigmae=fabs(sigmae);
fscanf(in,"%s",name);
fscanf(in,"%le",&sigmah);
sigmah=fabs(sigmah);
fscanf(in,"%s",name);
fscanf(in,"%le",&Nc);
Nc=fabs(Nc);
fscanf(in,"%s",name);
fscanf(in,"%le",&Nv);
Nv=fabs(Nv);
fscanf(in,"%s",name);
fscanf(in,"%le",&Tstart);
fscanf(in,"%s",name);
fscanf(in,"%le",&Tstop);
fscanf(in,"%s",name);
fscanf(in,"%d",&Tsteps);

fscanf(in,"%s",name);
fscanf(in,"%le",&nxstart);
fscanf(in,"%s",name);
fscanf(in,"%le",&nxsteps);
fscanf(in,"%s",name);
fscanf(in,"%le",&ndx);

fscanf(in,"%s",name);
fscanf(in,"%le",&pxstart);
fscanf(in,"%s",name);
fscanf(in,"%le",&pxsteps);
fscanf(in,"%s",name);
fscanf(in,"%le",&pdx);

printf("mue=%e\n",mue);
printf("muh=%e\n",muh);

fclose(in);
if (gendos==TRUE)
{
//pid_t pID = fork();
//if (pID == 0)
//{
gen_dos_fd_gaus_n(sigmae,Nc,Tstart,Tstop,Tsteps,bands,dband,traps,munrgstop,mue,alpha,nxstart,nxsteps,ndx,&confige);
//exit(0);
//}else
//{
gen_dos_fd_gaus_p(sigmah,Nv,Tstart,Tstop,Tsteps,bands,dband,traps,munrgstop,muh,alpha,pxstart,pxsteps,pdx,&configh);
//}
int status;
//waitpid(pID, &status, 0);
pick_dump();
printf("Finished generating....\n");
//exit(0);
}
//exit(0);
}


