// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include "hist.h"

/**Turn a histogram of separation distance in to a rdf by first normalizing then multiplying by volume and then successively dividing by the area of a sphere.
@param in the system from which the histogram came
@param hist the normalized histogram.
*/
void hist_make_rdf(double volume,struct hist *hist)
{
hist_norm(hist);
int i;
double r;
//printf("%lf\n",volume);
//getchar();
for (i=0;i<hist->x;i++)
{
r=hist->xscale[i];
hist->data[i][0]=volume*hist->data[i][0]/(4*3.1415926535849*r*r);
//printf("%lf\n",hist->data[i][0]);
}
hist->data[0][0]=0.0;
}

void hist_smoth(struct hist *myhist,int points)
{
int i=0;
int ii=0;
int pos=0;
double tot=0;
	for (i=0;i<myhist->x-points;i++)
	{
		for (ii=0;ii<points;ii++)
		{
			pos=i+ii;
			tot+=myhist->data[pos][0];
		}

		myhist->data[i][0]=(tot/(double)points);
		tot=0.0;
	}
}


void hist_wipe(struct hist *myhist)
{
int i=0;
int ii=0;
for (i=0;i<myhist->x;i++)
{
	for (ii=0;ii<myhist->y;ii++)
	{
		myhist->data[i][ii]=0.0;
	}
}
myhist->outofrange=0;
myhist->totpoints=0;
}

void hist_norm(struct hist *myhist)
{
int i;
//fprintf(out,"# points on plot= %d points which missed the plot = %d\n",myhist->totpoints,myhist->outofrange);
double tot=0;
double delta=myhist->xscale[1]-myhist->xscale[0];
	for (i=0;i<myhist->x;i++)
	{
		tot+=(double)myhist->data[i][0];
	}

	//tot*=myhist->xscale[myhist->x-1];
//tot=1.0;
	for (i=0;i<myhist->x;i++)
	{
		//if (myhist->data[i][ii]!=0)
		myhist->data[i][0]=((double)myhist->data[i][0])/(tot*delta);
		//don't print zeros
	}

}

void hist_dump_pdf(struct hist *myhist,char * name)
{
FILE* out;
out = fopen(name,"w");
int i;
fprintf(out,"# points on plot= %d points which missed the plot = %d\n",myhist->totpoints,myhist->outofrange);
double tot=0;
double delta=myhist->xscale[1]-myhist->xscale[0];
	for (i=0;i<myhist->x;i++)
	{
		tot+=(double)myhist->data[i][0];
	}

	//tot*=myhist->xscale[myhist->x-1];
//tot=1.0;
	for (i=0;i<myhist->x;i++)
	{
		//if (myhist->data[i][ii]!=0)
		fprintf(out,"%lf %lf\n",myhist->xscale[i],(((double)myhist->data[i][0])/(tot*delta)));
		//don't print zeros
	}

//test now......
//double my_tot=0.0;
//	for (i=1;i<myhist->x;i++)
//	{
//		my_tot+=delta*((double)myhist->data[i][0])/(tot*delta);
//	}
//printf("My tot %lf\n",my_tot);
//getchar();
}

void hist_div(struct hist *myhist,double div)
{
int i;
int ii;

for (ii=0;ii<myhist->y;ii++)
{
	for (i=0;i<myhist->x;i++)
	{
		myhist->data[i][ii]/=div;
	}

}

}

void hist_dump(struct hist *myhist,char * name)
{
FILE* out;
out = fopen(name,"w");
int i;
int ii;
fprintf(out,"# points on plot= %d points which missed the plot = %d\n",myhist->totpoints,myhist->outofrange);
for (ii=0;ii<myhist->y;ii++)
{
	for (i=0;i<myhist->x;i++)
	{
		if (myhist->data[i][ii]!=0)
		fprintf(out,"%lf %lf %e\n",myhist->xscale[i],myhist->yscale[ii],myhist->data[i][ii]);
		//don't print zeros
	}

fprintf(out,"\n");
}

}

void hist_mul(struct hist *myhist,double mul)
{
int i;
int ii;

for (ii=0;ii<myhist->y;ii++)
{
	for (i=0;i<myhist->x;i++)
	{
		myhist->data[i][ii]*=mul;
	}

}

}

void hist_dump2d(struct hist *myhist,char * name)
{
FILE* out;
out = fopen(name,"w");
int i;
fprintf(out,"# points on plot= %d points which missed the plot = %d\n",myhist->totpoints,myhist->outofrange);
	for (i=0;i<myhist->x;i++)
	{
		if (myhist->xstop!=-1)
		if (myhist->xscale[i]>myhist->xstop) break;

		//if (myhist->data[i][0]!=0)
		fprintf(out,"%lf %e\n",myhist->xscale[i],myhist->data[i][0]);
		//don't print zeros
	}


}



void hist_add(struct hist *myhist,double xvalue,double yvalue)
{
//printf("%lf\n",xvalue);
if ((xvalue>myhist->xmax)||(xvalue<myhist->xmin)||(yvalue>myhist->ymax)||(yvalue<myhist->ymin))
{

	myhist->outofrange++;
	return;
}
int xpos=(int)(((xvalue-myhist->xmin)/(myhist->xmax-myhist->xmin))*(double)myhist->x);
int ypos=(int)(((yvalue-myhist->ymin)/(myhist->ymax-myhist->ymin))*(double)myhist->y);
//	printf("%lf %lf -> %d %d\n",xvalue,yvalue,xpos, ypos);
//	getchar();

//protect memory over flow
if (xpos>=myhist->x)
{
printf("hist error x\n");
exit(0);
}

if (ypos>=myhist->y)
{
printf("hist error y\n");
exit(0);
}

myhist->data[xpos][ypos]+=1.0;
myhist->totpoints++;
}


void hist_init(struct hist *myhist, int x, int y,double xmin,double xmax,double ymin,double ymax)
{

myhist->x=x;
myhist->y=y;

myhist->xmin=xmin;
myhist->xmax=xmax;

myhist->ymin=ymin;
myhist->ymax=ymax;

int i;

myhist->xscale= (double *) malloc(x * sizeof(double));
myhist->yscale= (double *) malloc(y * sizeof(double));


myhist->data= (double **) malloc(x * sizeof(double **));

for (i=0;i<x;i++)
{
myhist->data[i]= (double *) malloc(y * sizeof(double));
}

for (i=0;i<x;i++)
{
myhist->xscale[i]=myhist->xmin+(myhist->xmax-myhist->xmin)*(((double)i)/((double)myhist->x));
}

for (i=0;i<y;i++)
{
myhist->yscale[i]=myhist->ymin+(myhist->ymax-myhist->ymin)*(((double)i)/((double)myhist->y));
}


hist_wipe(myhist);

myhist->xstop=-1;
}

void hist_free(struct hist *myhist)
{
int i;

for (i=0;i<myhist->x;i++)
{
	free(myhist->data[i]);
}

free(myhist->data);
free(myhist->xscale);
free(myhist->yscale);

}
